/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
//
//! \file   CAPTIVATE_PHONE_Demo.c
//
//  Group:          MSP
//  Target Devices: MSP430FR2633
//
//  (C) Copyright 2015, Texas Instruments, Inc.
//#############################################################################
// TI Release: 1.0.0.00
// Release Date: Friday, August 12, 2016 8:54 AM
//#############################################################################

//*****************************************************************************
// Includes
//*****************************************************************************

#include <msp430.h>
#include "captivate.h"
#include "CAPT_BSP.h"
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
#include "I2CMaster.h"
#include "DRV26x.h"
#endif
#include "CAPTIVATE_PHONE_Demo.h"

//*****************************************************************************
// Definitions
//*****************************************************************************

//! \brief Element ID of the speaker button in the mode button sensor
#define DEMO_SPEAKER_BTN        (3)

//! \brief Element ID of the mute button in the mode button sensor
#define DEMO_MUTE_BTN           (2)

//! \brief Element ID of the headset button in the mode button sensor
#define DEMO_HEADSET_BTN        (1)

//! \brief Element ID of the end call button in the mode button sensor
#define DEMO_ENDCALL_BTN        (0)

//*****************************************************************************
// Global Variables
//*****************************************************************************

//! \brief Flag set when guard channel is active
volatile bool Demo_guardMaskActive = false;

//! \brief Array of touch and previous touch states for numeric keypad buttons
volatile bool Demo_numericKeypadTouch[12] =
{false, false, false, false, false, false,false, false, false, false, false,
 false};
volatile bool Demo_numericKeypadPrevTouch[12] =
{false, false, false, false, false, false,false, false, false, false, false,
 false};

//! \brief Array of touch and previous touch states for mode keypad buttons
volatile bool Demo_modeKeypadTouch[4] = {false, false, false, false};
volatile bool Demo_modeKeypadPrevTouch[4] = {false, false, false, false};

//*****************************************************************************
// FRAM globals - Stored in FRAM to maintain state through power-loss
//*****************************************************************************

//! \brief Demo_volumeSliderPosition stores the last position of the volume slider
#pragma PERSISTENT(Demo_volumeSliderPosition)
#pragma LOCATION(Demo_volumeSliderPosition, 0x1800)
uint16_t Demo_volumeSliderPosition = 0;

//! \brief Demo_generalSliderPosition stores the last position of the general purpose slider
#pragma PERSISTENT(Demo_generalSliderPosition)
#pragma LOCATION(Demo_generalSliderPosition, 0x1802)
uint16_t Demo_generalSliderPosition = 0;

//! \brief Demo_headsetState stores the last state of the headset button
#pragma PERSISTENT(Demo_headsetState)
#pragma LOCATION(Demo_headsetState, 0x1804)
bool Demo_headsetState = false;

//! \brief Demo_muteState stores the last state of the mute button
#pragma PERSISTENT(Demo_muteState)
#pragma LOCATION(Demo_muteState, 0x1806)
bool Demo_muteState = false;

//! \brief Demo_speakerState stores the last state of the speaker button
#pragma PERSISTENT(Demo_speakerState)
#pragma LOCATION(Demo_speakerState, 0x1808)
bool Demo_speakerState = false;

//*****************************************************************************
// Function Prototypes
//*****************************************************************************

//! \brief The event handler for the numeric keypad sensor.
//! \param pSensor is a pointer to the calling sensor.
void Demo_numericKeypadHandler(tSensor* pSensor);

//! \brief The event handler for the mode keypad sensor.
//! \param pSensor is a pointer to the calling sensor.
void Demo_modeKeypadHandler(tSensor* pSensor);

//! \brief The event handler for the wheel button sensor.
//! \param pSensor is a pointer to the calling sensor.
void Demo_wheelButtonHandler(tSensor* pSensor);

//! \brief The event handler for the scroll wheel sensor.
//! \param pSensor is a pointer to the calling sensor.
void Demo_scrollWheelHandler(tSensor* pSensor);

//! \brief The event handler for the volume slider sensor.
//! \param pSensor is a pointer to the calling sensor.
void Demo_volumeSliderHandler(tSensor* pSensor);

//! \brief The event handler for the general purpose slider sensor.
//! \param pSensor is a pointer to the calling sensor.
void Demo_generalPurposeSliderHandler(tSensor* pSensor);

//! \brief The event handler for the proximity and guard sensor.
//! \param pSensor is a pointer to the calling sensor.
void Demo_guardChannelHandler(tSensor* pSensor);

//*****************************************************************************
// Function Implementations
//*****************************************************************************

void Demo_init(void)
{
    //
    // Open the I2C Master driver, which the DRV26x driver will use to
    // communicate with the DRV2605L haptic driver IC via I2C.
    // Enable the haptic driver by setting P1.0, which is connected
    // to the DRV2605L ENABLE pin.  Then, load the configuration for the
    // actuator, run an auto-calibration routine, set up for internal trigger
    // mode, and select the linear resonant actuator (LRA) effect library.
    //
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
    I2CMaster_open();
    P1OUT |= BIT0;
    DRV26x_reset();
    DRV26x_exitStandby();
    DRV26x_loadActuatorConfig(&DRV26x_actuator_DMJBRN1030);
    DRV26x_runAutoCalibration();
    DRV26x_setMode(DRV26x_mode_internalTrigger);
    DRV26x_selectEffectLibrary(DRV26x_lib_ROM_LRA);
#endif

    //
    // Associate the capacitive touch callback handlers with sensors.
    //
    MAP_CAPT_registerCallback(
        &numericKeypadSensor,
        &Demo_numericKeypadHandler
        );
    MAP_CAPT_registerCallback(
        &modeKeypadSensor,
        &Demo_modeKeypadHandler
        );
    MAP_CAPT_registerCallback(
        &wheelButtonSensor,
        &Demo_wheelButtonHandler
        );
    MAP_CAPT_registerCallback(
        &scrollWheelSensor,
        &Demo_scrollWheelHandler
        );
    MAP_CAPT_registerCallback(
        &volumeSliderSensor,
        &Demo_volumeSliderHandler
        );
    MAP_CAPT_registerCallback(
        &generalPurposeSliderSensor,
        &Demo_generalPurposeSliderHandler
        );
    MAP_CAPT_registerCallback(
        &proxAndGuardSensor,
        &Demo_guardChannelHandler
        );
}

void Demo_numericKeypadHandler(tSensor* pSensor)
{
    //
    // If the guard mask is activated, abort here and do not process
    // any events.
    //

    if(Demo_guardMaskActive == true)
    {
        return;
    }

    //check here for detect/touch

    uint8_t cycleIndex, elementIndex;
    for(cycleIndex = 0; cycleIndex < pSensor->ui8NrOfCycles; cycleIndex++)
    {
        for(elementIndex = 0;
            elementIndex < pSensor->pCycle[cycleIndex]->ui8NrOfElements;
            elementIndex++)
        {
            uint8_t keypadIndex = cycleIndex * 4 + elementIndex;

            Demo_numericKeypadPrevTouch[keypadIndex] =
                Demo_numericKeypadTouch[keypadIndex];
            Demo_numericKeypadTouch[keypadIndex] =
                pSensor->pCycle[cycleIndex]->pElements[elementIndex]->bTouch;

            if(Demo_numericKeypadTouch[keypadIndex] &&
               !Demo_numericKeypadPrevTouch[keypadIndex])
            {
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
                DRV26x_fireROMLibraryEffect(
                    DRV26x_effect_strongClick_100P,
                    true
                    );
#endif
            }
        }
    }
}

void Demo_modeKeypadHandler(tSensor* pSensor)
{
    //tButtonSensorParams *buttonParams;

    //
    // If the guard mask is activated, abort here and do not process
    // any events.
    //
    if(Demo_guardMaskActive == true)
    {
        return;
    }

    uint8_t cycleIndex, elementIndex;
    for(cycleIndex = 0; cycleIndex < pSensor->ui8NrOfCycles; cycleIndex++)
    {
        for(elementIndex = 0;
            elementIndex < pSensor->pCycle[cycleIndex]->ui8NrOfElements;
            elementIndex++)
        {
            uint8_t keypadIndex = cycleIndex * 4 + elementIndex;

            Demo_modeKeypadPrevTouch[keypadIndex] =
                Demo_modeKeypadTouch[keypadIndex];
            Demo_modeKeypadTouch[keypadIndex] =
                pSensor->pCycle[cycleIndex]->pElements[elementIndex]->bTouch;

            if(Demo_modeKeypadTouch[keypadIndex] &&
               !Demo_modeKeypadPrevTouch[keypadIndex])
            {
                switch(keypadIndex)
                {
                //
                // If the mute button was pressed, fire a double click.
                //
                case DEMO_MUTE_BTN:
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
                    DRV26x_fireROMLibraryEffect(
                        DRV26x_effect_doubleClick_100P,
                        true
                        );
#endif
                    // Enable write to FRAM data section
                    SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
                    Demo_muteState ^= true;
                    // Disable write to FRAM
                    SysCtl_protectFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
                    break;

                //
                // If the speaker button was pressed, fire a strong click.
                //
                case DEMO_SPEAKER_BTN:
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
                    DRV26x_fireROMLibraryEffect(
                        DRV26x_effect_strongClick_100P,
                        true
                        );
#endif
                    // Enable write to FRAM data section
                    SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
                    Demo_speakerState ^= true;
                    // Disable write to FRAM
                    SysCtl_protectFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
                    break;

                //
                // If the headset button was pressed, fire a strong click.
                //
                case DEMO_HEADSET_BTN:
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
                    DRV26x_fireROMLibraryEffect(
                        DRV26x_effect_strongClick_100P,
                        true
                        );
#endif
                    // Enable write to FRAM data section
                    SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
                    Demo_headsetState ^= true;
                    // Disable write to FRAM
                    SysCtl_protectFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
                    break;

                //
                // If the end call button was pressed, fire a triple click.
                //
                case DEMO_ENDCALL_BTN:
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
                    DRV26x_fireROMLibraryEffect(
                        DRV26x_effect_tripleClick_100P,
                        true
                        );
#endif
                    break;
                default:
                    break;
                }
            }
        }
    }
}

void Demo_wheelButtonHandler(tSensor* pSensor)
{
    //
    // If the guard mask is activated, abort here and do not process
    // any events.
    //
    if(Demo_guardMaskActive == true)
    {
        return;
    }

    //
    // If the sensor has a new touch, fire a "sharp pulsing" effect.
    //
    if((pSensor->bSensorTouch == true) && (pSensor->bSensorPrevTouch == false))
    {
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
        DRV26x_fireROMLibraryEffect(
            DRV26x_effect_pulsingSharp1_100P,
            true
            );
#endif
    }
}

void Demo_scrollWheelHandler(tSensor* pSensor)
{
    static uint16_t g_ui16PrevNumber = 0xFF;
    uint16_t ui16SensorPosition = 0;
    uint8_t ui8Number;
    tSliderSensorParams *pSliderParams;

    //
    // If the guard mask is activated, abort here and do not process
    // any events.
    //
    if(Demo_guardMaskActive == true)
    {
        return;
    }

    if(pSensor->bSensorTouch == true)
    {
        if(pSensor->bSensorPrevTouch == false)
        {
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
            DRV26x_fireROMLibraryEffect(
                DRV26x_effect_buzz1_100P,
                true
                );
#endif
        }
        else
        {
            pSliderParams = (tSliderSensorParams*)(pSensor->pSensorParams);
            for(ui8Number = 0; ui8Number < 10; ui8Number++)
            {
                ui16SensorPosition += 10;
                if(pSliderParams->SliderPosition.ui16Natural <
                   ui16SensorPosition)
                {
                    if(ui8Number != g_ui16PrevNumber)
                    {
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
                        DRV26x_fireROMLibraryEffect(
                            DRV26x_effect_softBump_100P,
                            false
                            );
#endif
                        g_ui16PrevNumber = ui8Number;
                    }
                    break;
                }
            }
        }
    }
}

void Demo_generalPurposeSliderHandler(tSensor* pSensor)
{
    static uint16_t g_ui16PrevNumber = 0xFF;
    uint16_t ui16SensorPosition = 0;
    uint8_t ui8Number;
    tSliderSensorParams *pSliderParams;

    //
    // If the guard mask is activated, abort here and do not process
    // any events.
    //
    if(Demo_guardMaskActive == true)
    {
        return;
    }

    if(pSensor->bSensorTouch == true)
    {
        pSliderParams = (tSliderSensorParams*)(pSensor->pSensorParams);
        if(pSensor->bSensorPrevTouch == false)
        {
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
            DRV26x_fireROMLibraryEffect(
                DRV26x_effect_buzz1_100P,
                true
                );
#endif
        }
        else
        {
            for(ui8Number = 0; ui8Number < 10; ui8Number++)
            {
                ui16SensorPosition += 10;
                if(pSliderParams->SliderPosition.ui16Natural <
                   ui16SensorPosition)
                {
                    if(ui8Number != g_ui16PrevNumber)
                    {
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
                        DRV26x_fireROMLibraryEffect(
                            DRV26x_effect_softBump_100P,
                            false
                            );
#endif
                        g_ui16PrevNumber = ui8Number;
                    }
                    break;
                }
            }
        }
        // Enable write to FRAM data section
        SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
        Demo_generalSliderPosition = pSliderParams->SliderPosition.ui16Natural;
        // Disable write to FRAM
        SysCtl_protectFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
    }
}

void Demo_volumeSliderHandler(tSensor* pSensor)
{
    static uint16_t g_ui16PrevNumber = 0xFF;
    uint16_t ui16SensorPosition = 0;
    uint8_t ui8Number;
    tSliderSensorParams *pSliderParams;

    //
    // If the guard mask is activated, abort here and do not process
    // any events.
    //
    if(Demo_guardMaskActive == true)
    {
        return;
    }

    if(pSensor->bSensorTouch == true)
    {
        pSliderParams = (tSliderSensorParams*)(pSensor->pSensorParams);
        if(pSensor->bSensorPrevTouch == false)
        {
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
            DRV26x_fireROMLibraryEffect(
                DRV26x_effect_buzz1_100P,
                true
                );
#endif
        }
        else
        {
            for(ui8Number = 0; ui8Number < 10; ui8Number++)
            {
                ui16SensorPosition += 10;
                if(pSliderParams->SliderPosition.ui16Natural <
                   ui16SensorPosition)
                {
                    if(ui8Number != g_ui16PrevNumber)
                    {
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
                        DRV26x_fireROMLibraryEffect(
                            DRV26x_effect_softBump_100P,
                            false
                            );
#endif
                        g_ui16PrevNumber = ui8Number;
                    }
                    break;
                }
            }
        }

        // Enable write to FRAM data section
        SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
        Demo_volumeSliderPosition = pSliderParams->SliderPosition.ui16Natural;
        // Disable write to FRAM
        SysCtl_protectFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
    }
}

void Demo_guardChannelHandler(tSensor *pSensor)
{
    //
    // If the guard channel is detecting a touch,
    // set the guard mask active flag to mask all other
    // touch processing.
    //
    if(pSensor->bSensorTouch == true)
    {
        Demo_guardMaskActive = true;
        if(pSensor->bSensorPrevTouch == false)
        {
#if  (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
            DRV26x_fireROMLibraryEffect(
                DRV26x_effect_smoothHum3_30P,
                false
                );
#endif
        }
    }
    //
    // If the guard channel is not detecting a touch,
    // clear the guard mask active flag to allow standard
    // touch processing.
    //
    else
    {
        Demo_guardMaskActive = false;
    }
}

void Demo_checkForValidTouch(void)
{
    uint8_t ui8Sensor;
    bool bTouch = false;

    //
    // If the guard mask is activated, clear LED1
    // and do not continue checking for a valid touch detection.
    // Indicate that the guard channel is active by illuminating LED2.
    //
    if(Demo_guardMaskActive == true)
    {
        LED1_OFF;
        LED2_ON;
        return;
    }

    //
    // If the guard mask is inactive, clear the guard LED.
    // Then, search the application's sensors for a valid touch detection.
    // If one is found, set the touch status flag.
    //
    LED2_OFF;
    for(ui8Sensor = 0; ui8Sensor < g_uiApp.ui8NrOfSensors; ui8Sensor++)
    {
        if((g_uiApp.pSensorList[ui8Sensor]->bSensorTouch == true) &&
           (g_uiApp.pSensorList[ui8Sensor] != &proxAndGuardSensor))
        {
            bTouch = true;
            break;
        }
    }

    //
    // If the touch status flag is set, also set LED1.
    //
    if(bTouch == true)
    {
        LED1_ON;
    }
    else
    {
        LED1_OFF;
    }
}
