/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
// TI Release: 2.0.0.00
// Release Date: Tuesday, December 20, 2016 2:00 PM
//#############################################################################
//*****************************************************************************
// CAPTHost_DataTable.h
//
// Data table used to keep track of the number of CapTIvate communications
// data packets transmitted via UART
//
// Created on: 11/20/2015
//
//*****************************************************************************

//*****************************************************************************
//! \addtogroup CAPT_DataTable
//! @{
//*****************************************************************************

#ifndef CAPT_METADATA_H_
#define CAPT_METADATA_H_

//****************************************************************************
// Includes
//****************************************************************************
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include "CAPTHost_CommConfig.h"

//! \brief Defines the code returned when requested data is not
//!        found in the data table
#define DATATABLE_PACKET_NOT_IN_TABLE_CODE              (0xFF)

//****************************************************************************
// Typedefs
//****************************************************************************

//*****************************************************************************
//
//! typedef DataTable_Error is the enumeration of error codes that can
//! be returned by data table functions
//
//*****************************************************************************
typedef enum
{
    //
    //! DataTable_NoError indicates no error was detected during a function
    //! execution
    //
    DataTable_NoError = 0,

    //
    //! DataTable_MemcpyError indicates an error occured while copying data
    //! into the data table
    //
    DataTable_MemcpyError = 1,

    //
    //! DataTable_DataPacketLimitError indicates that the total number of
    //! unique data packets that can be stored in the table has been exceeded
    //
    DataTable_DataPacketLimitError = 2,

    //
    //! DataTable_DataByteLimitError indicates that the total number of bytes
    //! that can be stored in the data table has been exceeded
    //
    DataTable_DataByteLimitError = 3
} DataTable_Error;

//*****************************************************************************
//
//! typedef DataTable_PacketType is the enumeration of packet types that can
//! be stored in the data table
//
//*****************************************************************************
typedef enum
{
    //
    //! DataTable_SensorPacketType indicates the data is a CapTIvate&tm;
    //! data packet
    //
    DataTable_SensorPacketType,

    //
    //! DataTable_CyclePacketType indicates that data is a CapTIvate&tm;
    //! cycle packet
    //
    DataTable_CyclePacketType,

    //
    //! DataTable_GeneralPacketType indicates that data is a CapTIvate&tm;
    //! general purpose data packet
    //
    DataTable_GeneralPacketType
} DataTable_PacketType;

//*****************************************************************************
//
//! typedef DataTable_LookupTableEntry holds the metadata information for a
//! single packet in the data table.
//
//*****************************************************************************
typedef struct
{
    //
    //! type is the type of the data packet
    //
    DataTable_PacketType type;

    //
    //! sensorID is the ID of the sensor associated with the data packet
    //
    uint8_t sensorID;

    //
    //! cycleID is the ID of the cycle associated with the data
    //
    uint8_t cycleID;

    //
    //! fresh indicates if the current data stored has been read or not
    //
    bool fresh;

    //
    //! *dataPtr is a pointer to the location of the packet in the data table
    //
    uint8_t *dataPtr;
} DataTable_LookupTableEntry;

//*****************************************************************************
//
//! typedef DataTable_Metadata is the metadata for the data table. It allows
//! the table to be maintained and quickly searched.
//
//*****************************************************************************
typedef struct
{
    //
    //! totalDataEntries it the total number of unique data packets currently
    //! stored in the table
    //
    volatile uint8_t totalDataEntries;

    //
    //! *nextAvailableLocation is the next location in the table that new data
    //! can be added
    //
    uint8_t *nextAvailableLocation;

    //
    //! *endOfData is the last location in memory that data in the data table
    //! can be stored
    //
    uint8_t *endOfData;

    //
    //! lookUpTable is the table of information about packets currently stored
    //! in the data table
    //
    DataTable_LookupTableEntry lookUpTable[MAX_NUMBER_OF_PACKETS];
} DataTable_Metadata;

//*****************************************************************************
//
//! typedef DataTable_Type is the type of a data table and includes metadata
//! and a continuous section of data bytes.
//
//*****************************************************************************
typedef struct
{
    //
    //! *data is the pointer to the start of the data packets in memory
    //
    uint8_t *data;

    //
    //! *metadata is the pointer to the metadata for the data table
    //
    DataTable_Metadata *metadata;
} DataTable_Type;

//****************************************************************************
// Function Prototypes
//****************************************************************************

//****************************************************************************
//
//! Initializes metadata for CapTIvate comms packet table
//!
//! \param table - The data table structure
//! \param dataArray - Array in which to store data
//! \param meta - Metadata for data table
//! \param dataArraySize - Size of the data array in bytes
//!
//! \return none
//
//****************************************************************************
void CAPT_initDataTable(DataTable_Type *table,
                        DataTable_Metadata *meta,
                        uint8_t *dataArray,
                        uint16_t dataArraySize);

//****************************************************************************
//
//! Handles updating the table in response to receiving a data packet
//!
//! \param table - The table in which received packets are stored
//! \param packet - The received data packet
//! \param type - The type of the data packet
//! \param sensorID - ID of the sensor associated with the packet
//! \param cycleID - ID of the cycle associated with packet (could be NULL)
//! \param size - Size of the data packet in bytes
//!
//! \return Error code generated while updating table
//
//****************************************************************************
DataTable_Error CAPT_putReceivedPacketInDataTable(DataTable_Type *table,
                                                  void *packet,
                                                  DataTable_PacketType type,
                                                  uint8_t sensorID,
                                                  uint8_t cycleID,
                                                  size_t size);

//****************************************************************************
//
//! Adds a new packet to the data table
//!
//! \param table - The data table to which to add the packet
//! \param packet - The data packet to store
//! \param type - Data packet type
//! \param sensorID - ID of the sensor that data is associated with
//! \param cycleID - ID of the cycle that data is associated with (could be NULL)
//! \param size - Size of data packet in bytes
//!
//! \return Error code generated while adding new packet to table
//
//****************************************************************************
DataTable_Error CAPT_addNewPacketToDataTable(DataTable_Type *table,
                                             void *packet,
                                             DataTable_PacketType type,
                                             uint8_t sensorID,
                                             uint8_t cycleID,
                                             size_t size);

//****************************************************************************
//
//! Updates the packet if it already exists in the data table
//!
//! \param table - The data table to store the packet in
//! \param tableLocation - Place in lookup table of metadata where packet info is currently stored
//! \param packet - The data packet to store
//! \param size - The size of the data in bytes
//!
//! \return Error code generated during update
//
//****************************************************************************
DataTable_Error CAPT_updatePacketInDataTable(DataTable_Type *table,
                                             uint8_t tableLocation,
                                             void *packet,
                                             size_t size);

//****************************************************************************
//
//! \brief Checks metadata of data table for a specific packet
//!
//! \param table - Data table to check
//! \param type - Type of packet to search for
//! \param sensorID - ID of sensor packet with which packet is associated
//! \param cycleID - ID of cycle with which packet is associated
//!
//! \returns Lookup table location for packet
//
//****************************************************************************
uint8_t CAPT_checkForPacketInDataTable(DataTable_Type *table,
                                       DataTable_PacketType type,
                                       uint8_t sensorID,
                                       uint8_t cycleID);

#endif /* CAPT_METADATA_H_ */

//*****************************************************************************
// Close the Doxygen group.
//! @}
//*****************************************************************************
