/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//****************************************************************************************
//
//! \file   main.c
//!
//! \brief  This demo displays the CapTIvate IP Phone panel UI on a Kentec LCD display
//!         and updates the display according to received touch data.
//!
//!         When the guard channel is inactive and a valid touch is detected,
//!         the display will show the touch. A yellow outline will appear around the UI
//!         when proximity detection is made on the guard channel/proximity sensor.
//!         When the guard channel is activated, a notification will be displayed and all
//!         other inputs on the UI will be locked.
//!         After the guard channel is returned to an inactive state, the display is
//!         refreshed and the application continues.
//
//#############################################################################
// TI Release: 2.0.0.00
// Release Date: Tuesday, December 20, 2016 2:00 PM
//#############################################################################
//*****************************************************************************
// Includes
//*****************************************************************************
#include <stdint.h>
#include <stdbool.h>
#include <Timer.h>

#include "driverlib.h"
#include "grlib.h"
#include "imageButton.h"
#include "LCD.h"
#include "Demo.h"

#include "CAPTHost_CommConfig.h"
#include "CAPTHost_SensorConfig.h"
#include "CAPTHost_Interface.h"

//*****************************************************************************
// Globals
//*****************************************************************************
Demo_ButtonTouchType numKeypadButtons[12];
Demo_ButtonTouchType modeKeypadButtons[4];
Demo_ButtonTouchType wheelButton;
Demo_SliderAndWheelType volumeSlider = { 0, 0 };
Demo_SliderAndWheelType generalPurposeSlider = { 0, 0 };
Demo_SliderAndWheelType scrollWheel = { 0, 0 };
Demo_GuardChannelState guardChannel = { false, false, false, false };
Demo_StateType demoState = Demo_UpdateUI;

//*****************************************************************************
// Function Prototypes
//*****************************************************************************
void System_init(void);
void Demo_updateNumKeypad(void);
void Demo_updateModeKeypad(void);
void Demo_updateVolumeSlider(void);
void Demo_updateGeneralPurposeSlider(void);
void Demo_updateScrollWheel(void);
void Demo_updateWheelButton(void);
void Demo_updateGuardChannel(void);
void Demo_sleepUntilRefresh(uint8_t delay);

void main(void)
{
    // Stop watchdog timer
	MAP_WDT_A_holdTimer();

    // Initialize the system, setup LCD, update display, initialize CapTIvate
	//interface
    System_init();
    LCD_init();
    CAPT_initCommInterface();

    // Draw last known state
    LCD_drawSavedState(&modeKeypadButtons[0], &volumeSlider,
                       &generalPurposeSlider);

    // Loop to detect touch
    while(1)
    {
        Demo_sleepUntilRefresh(2);
#if (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
        // Check to see if new data has been received via UART
        CAPT_checkForInboundPacket();
        if(TimerA1_count == 0)
        {
#endif

        // Respond to sensor data
        Demo_updateGuardChannel();
        if(demoState == Demo_UpdateUI)
        {
            // Update UI
            Demo_updateNumKeypad();
            Demo_updateModeKeypad();
            Demo_updateVolumeSlider();
            Demo_updateWheelButton();
            Demo_updateGeneralPurposeSlider();
            Demo_updateScrollWheel();
        }
        else if(demoState == Demo_ReleaseUI)
        {
            // Redraw last valid state
            LCD_drawSavedState(&modeKeypadButtons[0], &volumeSlider,
                               &generalPurposeSlider);
            demoState = Demo_UpdateUI;
        }

#if (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
    }
#endif
    }
}

void System_init(void)
{
    // Enable FPU for setting DCO frequency
    MAP_FPU_enableModule();

    // 2 flash wait states, VCORE = 1, running off DC-DC, 48 MHz
    MAP_FlashCtl_setWaitState(FLASH_BANK0, 2);
    MAP_FlashCtl_setWaitState(FLASH_BANK1, 2);
    MAP_PCM_setPowerState(PCM_AM_DCDC_VCORE1);
    MAP_CS_setDCOCenteredFrequency(CS_DCO_FREQUENCY_48);
    MAP_CS_setDCOFrequency(CS_48MHZ);
    MAP_CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, 1);
    MAP_CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, 1);
    MAP_CS_initClockSignal(CS_HSMCLK, CS_DCOCLK_SELECT, 1);

    // Setup pins for UART
    MAP_GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P3, GPIO_PIN3,
                                                GPIO_PRIMARY_MODULE_FUNCTION);
    MAP_GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P3, GPIO_PIN2,
                                               GPIO_PRIMARY_MODULE_FUNCTION);

    // Setup pins for I2C
    MAP_GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P6,GPIO_PIN4,
                                                GPIO_PRIMARY_MODULE_FUNCTION);
    MAP_GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P6,GPIO_PIN5,
                                                GPIO_PRIMARY_MODULE_FUNCTION);

    // Globally enable interrupts.
    __enable_interrupt();

    // Enable Timer A1 interrupt for handling sleep cycles
    NVIC_EnableIRQ(TA1_0_IRQn);

#if (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
    // Enable EUSCIA2 interrupt for UART comms
    NVIC_EnableIRQ(EUSCIA2_IRQn);
#elif (CAPT_INTERFACE == __CAPT_I2C_INTERFACE__)
    // Enable EUSCIB1 interrupt for I2C comms
    NVIC_EnableIRQ(EUSCIB1_IRQn);
#endif
}

void Demo_updateNumKeypad(void)
{
    // Save previous state of buttons
    uint8_t buttonIndex;
    for(buttonIndex = 0; buttonIndex < 12; buttonIndex++)
    {
        numKeypadButtons[buttonIndex].prevTouch =
            numKeypadButtons[buttonIndex].touch;
    }

    // Read touch state of buttons on cycle 0
    CyclePacket numericKeypadCycle0, numericKeypadCycle1, numericKeypadCycle2;
    if(CAPT_getCyclePacket(&numericKeypadCycle0, NUMERIC_KEYPAD_SENSOR_ID, 0,
                           NUMERIC_KEYPAD_CYCLE0_ELEMENTS))
    {
        numKeypadButtons[0].touch =
            numericKeypadCycle0.cyclePacketBase->element1Touch;
        numKeypadButtons[1].touch =
            numericKeypadCycle0.cyclePacketBase->element2Touch;
        numKeypadButtons[2].touch =
            numericKeypadCycle0.cyclePacketBase->element3Touch;
        numKeypadButtons[3].touch =
            numericKeypadCycle0.cyclePacketBase->element4Touch;
    }

    // Read touch state of buttons on cycle 1
    if(CAPT_getCyclePacket(&numericKeypadCycle1, NUMERIC_KEYPAD_SENSOR_ID, 1,
                           NUMERIC_KEYPAD_CYCLE1_ELEMENTS))
    {
        numKeypadButtons[4].touch =
            numericKeypadCycle1.cyclePacketBase->element1Touch;
        numKeypadButtons[5].touch =
            numericKeypadCycle1.cyclePacketBase->element2Touch;
        numKeypadButtons[6].touch =
            numericKeypadCycle1.cyclePacketBase->element3Touch;
        numKeypadButtons[7].touch =
            numericKeypadCycle1.cyclePacketBase->element4Touch;
    }

    // Read touch state of buttons on cycle 2
    if(CAPT_getCyclePacket(&numericKeypadCycle2, NUMERIC_KEYPAD_SENSOR_ID, 2,
                           NUMERIC_KEYPAD_CYCLE2_ELEMENTS))
    {
        numKeypadButtons[8].touch =
            numericKeypadCycle2.cyclePacketBase->element1Touch;
        numKeypadButtons[9].touch =
            numericKeypadCycle2.cyclePacketBase->element2Touch;
        numKeypadButtons[10].touch =
            numericKeypadCycle2.cyclePacketBase->element3Touch;
        numKeypadButtons[11].touch =
            numericKeypadCycle2.cyclePacketBase->element4Touch;
    }

    // Update button images
    LCD_drawNumKeypad(&numKeypadButtons[0]);
}

void Demo_updateModeKeypad(void)
{
    // Read newest touch value
    CyclePacket modeKeypadCycle0;
    if(CAPT_getCyclePacket(&modeKeypadCycle0, MODE_KEYPAD_SENSOR_ID, 0,
                           MODE_KEYPAD_CYCLE0_ELEMENTS))
    {
        // Save previous state
        uint8_t index;
        for(index = 0; index < 4; index++)
        {
            modeKeypadButtons[index].prevTouch = modeKeypadButtons[index].touch;
        }

        // Read new state
        modeKeypadButtons[0].touch =
            modeKeypadCycle0.cyclePacketBase->element1Touch;
        modeKeypadButtons[1].touch =
            modeKeypadCycle0.cyclePacketBase->element2Touch;
        modeKeypadButtons[2].touch =
            modeKeypadCycle0.cyclePacketBase->element3Touch;
        modeKeypadButtons[3].touch =
            modeKeypadCycle0.cyclePacketBase->element4Touch;

        // Update button images
        LCD_drawModeKeypad(&modeKeypadButtons[0]);
    }
}

void Demo_updateVolumeSlider(void)
{
    // Read slider data
    SensorData *volumeSliderSensor = CAPT_getSensorPacket(
        VOLUME_SLIDER_SENSOR_ID);

    // Verify data was found
    if(volumeSliderSensor != NULL)
    {
        // Read position from data
        uint16_t position =
            ((uint16_t)volumeSliderSensor->byte1 <<
        8) + volumeSliderSensor->byte0;

        // Update Position Indicator on LCD
        if(position != 0xFFFF && position != volumeSlider.position)
        {
            volumeSlider.prevPosition = volumeSlider.position;
            volumeSlider.position = position;
            LCD_drawVolumeSlider(&volumeSlider);
        }
    }
}

void Demo_updateGeneralPurposeSlider(void)
{
    // Read general purpose slider data
    SensorData *generalPurposeSliderSensor = CAPT_getSensorPacket(
        GENERAL_PURPOSE_SLIDER_SENSOR_ID);

    if(generalPurposeSliderSensor != NULL)
    {
        uint16_t position =
            ((uint16_t)generalPurposeSliderSensor->byte1 <<
            8) + generalPurposeSliderSensor->byte0;

        // Update Position Indicator on LCD
        if(position != 0xFFFF && position != generalPurposeSlider.position)
        {
            generalPurposeSlider.prevPosition = generalPurposeSlider.position;
            generalPurposeSlider.position = position;
            LCD_drawGPSlider(&generalPurposeSlider);
        }
    }
}

void Demo_updateScrollWheel(void)
{
    // Read scroll wheel data
    SensorData *scrollWheelSensor =
        CAPT_getSensorPacket(SCROLL_WHEEL_SENSOR_ID);

    // Verify data was found
    if(scrollWheelSensor != NULL)
    {
        scrollWheel.prevPosition = scrollWheel.position;
        scrollWheel.position =
            ((uint16_t)scrollWheelSensor->byte1 <<
        8) + scrollWheelSensor->byte0;
        LCD_drawScrollWheel(&scrollWheel);
    }
}

void Demo_updateWheelButton(void)
{
    // Read wheel button data
    SensorData *wheelButtonSensor =
        CAPT_getSensorPacket(WHEEL_BUTTON_SENSOR_ID);

    // Verify data was found
    if(wheelButtonSensor != NULL)
    {
        wheelButton.prevTouch = wheelButton.touch;
        wheelButton.touch = wheelButtonSensor->touchFlag;
        LCD_drawWheelButton(&wheelButton);
    }
}

void Demo_updateGuardChannel(void)
{
    guardChannel.prevTouch = guardChannel.touch;
    guardChannel.prevProximity = guardChannel.proximity;

    // Read guard channel data
    SensorData *guardChannelSensor = CAPT_getSensorPacket(
        PROX_AND_GUARD_SENSOR_ID);

    // Verify data was found
    if(guardChannelSensor != NULL)
    {
        // Update current state if new data found
        guardChannel.touch = guardChannelSensor->touchFlag;
        guardChannel.proximity = guardChannelSensor->proximityFlag;
        LCD_drawGuardChannel(&guardChannel);
    }

    if(guardChannel.touch)
    {
        demoState = Demo_HoldUI;
    }
    else if(!guardChannel.touch && guardChannel.prevTouch)
    {
        demoState = Demo_ReleaseUI;
    }
}

void Demo_sleepUntilRefresh(uint8_t delay)
{
    // Sleep for given number of ACLK/8 cycles
    TimerA1_sleep(delay);
    while(TimerA1_flag == false)
    {
        __sleep();
    }
    TimerA1_flag = false;
}
