/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
// TI Release: 2.0.0.00
// Release Date: Tuesday, December 20, 2016 2:00 PM
//#############################################################################


//*****************************************************************************
// CAPT_ByteQueue.c
//*****************************************************************************
#include <CAPTHost_ByteQueue.h>
#include <stdint.h>
#include <stdlib.h>

bool CAPT_initByteQueue(ByteQueue_Type *queue,
                        uint8_t *bufferArray,
                        uint16_t bufferArraySize)
{
    //
    // If buffer pointer or queue pointer are invalid, return false
    //
    if((bufferArray == 0) || (queue == 0))
    {
        return(false);
    }

    //
    // If buffer size is less than 2, a queue cannot be created! Return false.
    //
    if(bufferArraySize < 2)
    {
        return(false);
    }

    //
    // Set tail and head indexes to zero, and store off buffer pointer
    // and buffer size.
    //
    queue->queueHead = 0;
    queue->queueTail = 0;
    queue->queueArray = bufferArray;
    queue->queueArraySize = bufferArraySize;

    return(true);
}

int8_t CAPT_pushOntoByteQueue(ByteQueue_Type *queue,
                              uint8_t dataToAdd)
{
    int8_t returnStatus = ByteQueue_Success;

    //
    // Check for queue full case (overwrite oldest data point)
    //
    if(CAPT_isByteQueueFull(queue) == true)
    {
        queue->queueArray[queue->queueHead] = 0;
        queue->queueHead++;
        if(queue->queueHead == queue->queueArraySize)
        {
            queue->queueHead = 0;
        }
        returnStatus = ByteQueue_QueueOverrun;
    }

    //
    // Add item to queue
    //
    queue->queueArray[queue->queueTail++] = dataToAdd;

    //
    // Check for wrap-around case (ring)
    // Full case has already been checked above
    //
    if(queue->queueTail == queue->queueArraySize)
    {
        queue->queueTail = 0;
    }

    return(returnStatus);
}

int8_t CAPT_pullFromByteQueue(ByteQueue_Type *queue,
                              uint8_t *destination)
{
    //
    // Check for empty queue case (no item to pull)
    //
    if(CAPT_isByteQueueEmpty(queue) == true)
    {
        return(ByteQueue_QueueEmpty);
    }

    //
    // Get data from queue head and move head index
    //
    *destination = queue->queueArray[queue->queueHead];
    queue->queueArray[queue->queueHead] = 0;
    queue->queueHead++;

    //
    // Check for wrap-around case (ring)
    //
    if(queue->queueHead == queue->queueArraySize)
    {
        queue->queueHead = 0;
    }

    return(ByteQueue_Success);
}

uint16_t CAPT_getByteQueueSize(ByteQueue_Type *queue)
{
    uint16_t size = 0;

    //
    // Check for empty case (0)
    //
    if(CAPT_isByteQueueEmpty(queue) == true)
    {
        return(0);
    }

    //
    // Check for standard case (head is follows tail)
    //
    if(queue->queueTail > queue->queueHead)
    {
        size = queue->queueTail - queue->queueHead;
    }
    //
    // Wrap-around case - Head is greater than tail
    //
    else
    {
        size = queue->queueArraySize - queue->queueHead
               + queue->queueTail;
    }

    return(size);
}

bool CAPT_isByteQueueFull(ByteQueue_Type *queue)
{
    //
    // Check for tail approaching head full case
    //
    if(queue->queueHead != 0)
    {
        if(queue->queueTail == (queue->queueHead - 1))
        {
            return(true);
        }
    }
    //
    // Check wrap-around full case
    //
    else if(queue->queueTail == (queue->queueArraySize - 1))
    {
        return(true);
    }

    return(false);
}

bool CAPT_isByteQueueEmpty(ByteQueue_Type *queue)
{
    //
    // queue is empty when head index and tail index are the same.
    //
    if(queue->queueTail == queue->queueHead)
    {
        return(true);
    }

    return(false);
}
