/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
// I2CMaster.c
//
// \version 0.03.00.00
// Released on Thu, Aug 27, 2015  4:24:33 PM
//
//*****************************************************************************

#include "I2CMaster.h"

#if (I2CMASTER__ENABLE == true)

//*****************************************************************************
//
//! \typedef tI2CMasterState defines the states that the I2C master
//! driver may be in during operation.
//
//*****************************************************************************
typedef enum
{
    I2CMasterIsClosed = 0,
    I2CMasterIsIdle,
    I2CMasterIsCommunicating
} I2CMasterState;

typedef union
{
    uint8_t allFlags;
    struct
    {
        uint8_t errorFlags : 2;
        uint8_t statusFlags : 2;
    };
    struct
    {
        uint8_t slaveNack : 1;
        uint8_t clockLowTimeout : 1;
        uint8_t stop : 1;
        uint8_t start : 1;
    };
} I2CMasterFlags;

typedef struct
{
    I2CMasterState state;
    I2CMasterFlags flags;
    uint8_t *data;
} I2CMasterDriver;

//*****************************************************************************
//
// eUSCI_B I2C Registers
//
//*****************************************************************************
#if defined (__MSP432P401R__)
#define I2CMASTER__CTLW0                                                      \
    HWREG16(I2CMASTER__EUSCI_B_PERIPHERAL + OFS_UCB1CTLW0)

#define I2CMASTER__CTLW1                                                      \
    HWREG16(I2CMASTER__EUSCI_B_PERIPHERAL + OFS_UCB1CTLW1)

#define I2CMASTER__BRW                                                        \
    HWREG16(I2CMASTER__EUSCI_B_PERIPHERAL + OFS_UCB1BRW)

#define I2CMASTER__TBCNT                                                      \
    HWREG16(I2CMASTER__EUSCI_B_PERIPHERAL + OFS_UCB1TBCNT)

#define I2CMASTER__I2CSA                                                      \
    HWREG16(I2CMASTER__EUSCI_B_PERIPHERAL + OFS_UCB1I2CSA)

#define I2CMASTER__STATW                                                      \
    HWREG16(I2CMASTER__EUSCI_B_PERIPHERAL + OFS_UCB1STATW)

#define I2CMASTER__IE                                                         \
    HWREG16(I2CMASTER__EUSCI_B_PERIPHERAL + OFS_UCB1IE)

#define I2CMASTER__IFG                                                        \
    HWREG16(I2CMASTER__EUSCI_B_PERIPHERAL + OFS_UCB1IFG)

#define I2CMASTER__TXBUF                                                      \
    HWREG16(I2CMASTER__EUSCI_B_PERIPHERAL + OFS_UCB1TXBUF)

#define I2CMASTER__RXBUF                                                      \
    HWREG16(I2CMASTER__EUSCI_B_PERIPHERAL + OFS_UCB1RXBUF)
#endif

//*****************************************************************************
//
//! eUSCI_B I2C Shortcut Macros
//
//*****************************************************************************
#define I2CMASTER__CLEAR_RST                     (I2CMASTER__CTLW0 &= ~UCSWRST)
#define I2CMASTER__SET_RST                        (I2CMASTER__CTLW0 |= UCSWRST)
#define I2CMASTER__WRITEMODE                         (I2CMASTER__CTLW0 |= UCTR)
#define I2CMASTER__READMODE                         (I2CMASTER__CTLW0 &= ~UCTR)
#define I2CMASTER__START                          (I2CMASTER__CTLW0 |= UCTXSTT)
#define I2CMASTER__RESTART                                    (I2CMASTER_START)
#define I2CMASTER__STOP                           (I2CMASTER__CTLW0 |= UCTXSTP)

//*****************************************************************************
//
//! \var g_state specifies the current state of the driver.
//
//*****************************************************************************
static volatile I2CMasterDriver driver =
{
    .state = I2CMasterIsClosed
};

static I2CMasterError I2CMaster_checkForErrors(void);

static inline void I2CMaster_prep(uint8_t addr,
                                  uint8_t *data,
                                  uint8_t cnt);

static inline void I2CMaster_cleanup(void);

static inline I2CMasterError I2CMaster_finish(void);

void I2CMaster_open(const I2C_PortType *port)
{
    I2CMASTER__SET_RST;
    I2C_initMaster(I2CMASTER__EUSCI_B_PERIPHERAL, &port->peripheralParameters);
    driver.state = I2CMasterIsIdle;
}

void I2CMaster_close(void)
{
    I2CMASTER__SET_RST;
    driver.state = I2CMasterIsClosed;
}

I2CMasterError I2CMaster_checkForErrors(void)
{
    if(driver.flags.clockLowTimeout == true)
    {
        return(I2CMaster_busTimeoutError);
    }
    else if(driver.flags.slaveNack == true)
    {
        return(I2CMaster_slaveNackError);
    }
    else
    {
        return(I2CMaster_noError);
    }
}

inline void I2CMaster_prep(uint8_t addr,
                           uint8_t *data,
                           uint8_t cnt)
{
    I2CMASTER__SET_RST;
    driver.state = I2CMasterIsCommunicating;
    driver.flags.allFlags = 0;
    driver.data = data;
    I2CMASTER__I2CSA = addr;
    I2CMASTER__TBCNT = cnt;
    I2CMASTER__CLEAR_RST;
    I2CMASTER__IE = UCNACKIE | UCCLTOIE | UCSTPIFG;
}

inline void I2CMaster_cleanup(void)
{
    I2CMASTER__IE = 0;
    driver.state = I2CMasterIsIdle;
}

inline I2CMasterError I2CMaster_finish(void)
{
    while(driver.flags.stop == false)
    {
        if(driver.flags.errorFlags != 0)
        {
            I2CMaster_cleanup();
            return(I2CMaster_checkForErrors());
        }
    }

    I2CMaster_cleanup();
    return(I2CMaster_checkForErrors());
}

I2CMasterError I2CMaster_writeBuffer(uint8_t addr,
                                     uint8_t *data,
                                     uint8_t cnt)
{
    if(driver.state != I2CMasterIsIdle)
    {
        return(I2CMaster_unavailableError);
    }

    I2CMaster_prep(addr, data, cnt);

    I2CMASTER__WRITEMODE;
    I2CMASTER__START;
    while(!(I2CMASTER__IFG & UCTXIFG))
    {
        if(driver.flags.errorFlags != 0)
        {
            I2CMaster_cleanup();
            return(I2CMaster_checkForErrors());
        }
    }

    // Begin writing via ISR
    I2CMASTER__IE |= UCTXIE;

    return(I2CMaster_finish());
}

I2CMasterError I2CMaster_readBuffer(uint8_t addr,
                                    uint8_t *data,
                                    uint8_t cnt)
{
    if(driver.state != I2CMasterIsIdle)
    {
        return(I2CMaster_unavailableError);
    }

    I2CMaster_prep(addr, data, cnt);
    I2CMASTER__READMODE;
    I2CMASTER__START;
    I2CMASTER__IE |= UCRXIE;

    return(I2CMaster_finish());
}

//*****************************************************************************
//
//! \def I2CMASTER__EUSCI_VECTOR defines the interrupt vector location
//! of the selected eUSCI_B I2C peripheral.
//
//! \def I2CMASTER__EUSCI_IV defines the interrupt vector of the selected
//! eUSCI_B I2C peripheral.
//
//*****************************************************************************
#if defined (__MSP432P401R__)
void I2CMaster_ISR(void)
{
    if(I2CMASTER__IFG & UCALIFG)
    {
        I2CMASTER__IFG &= ~UCALIFG;
    }
    else if(I2CMASTER__IFG & UCNACKIFG)
    {
        I2CMASTER__IFG &= ~UCNACKIFG;
        I2CMASTER__STOP;
        driver.flags.slaveNack = true;
    }
    else if(I2CMASTER__IFG & UCSTTIFG)
    {
        I2CMASTER__IFG &= ~(UCSTTIFG);
        driver.flags.start = true;
    }
    else if(I2CMASTER__IFG & UCSTPIFG)
    {
        I2CMASTER__IFG &= ~(UCSTPIFG);
        driver.flags.stop = true;
    }
    else if(I2CMASTER__IFG & UCRXIFG0)
    {
        *driver.data++ = I2CMASTER__RXBUF;
    }
    else if(I2CMASTER__IFG & UCTXIFG0)
    {
        I2CMASTER__TXBUF = *driver.data++;
    }
    else if(I2CMASTER__IFG & UCBCNTIFG)
    {
        I2CMASTER__IFG &= ~UCBCNTIFG;
    }
    else if(I2CMASTER__IFG & UCCLTOIFG)
    {
        I2CMASTER__IFG &= ~UCCLTOIFG;
        driver.flags.clockLowTimeout = true;
    }

    __low_power_mode_off_on_exit();
}

#endif

#endif /* I2CMASTER__ENABLE==true */
