/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
// TI Release: 1.0.0.00
// Release Date: Friday, August 12, 2016 8:54 AM
//#############################################################################
//*****************************************************************************
// CAPTHost_DataTable.c
//*****************************************************************************

#include <CAPTHost_DataTable.h>
#include <stdlib.h>

void CAPT_initDataTable(DataTable_Type *table,
                        DataTable_Metadata *meta,
                        uint8_t *dataArray,
                        uint16_t dataArraySize)
{
    // Initialize metadata
    meta->totalDataEntries = 0;
    meta->nextAvailableLocation = dataArray;
    meta->endOfData = &dataArray[dataArraySize - 1];

    // Assign metadata and data array to table
    table->metadata = meta;
    table->data = dataArray;
}

DataTable_Error CAPT_putReceivedPacketInDataTable(DataTable_Type *table,
                                                  void *packet,
                                                  DataTable_PacketType type,
                                                  uint8_t sensorID,
                                                  uint8_t cycleID,
                                                  size_t size)
{
    DataTable_Error error = DataTable_NoError;

    uint8_t tableLocation = CAPT_checkForPacketInDataTable(table, type,
                                                           sensorID, cycleID);

    if(tableLocation == DATATABLE_PACKET_NOT_IN_TABLE_CODE)
    {
        error = CAPT_addNewPacketToDataTable(table, packet, type, sensorID,
                                             cycleID, size);
    }
    else
    {
        error =
            CAPT_updatePacketInDataTable(table, tableLocation, packet, size);
    }

    return(error);
}

DataTable_Error CAPT_addNewPacketToDataTable(DataTable_Type *table,
                                             void *packet,
                                             DataTable_PacketType type,
                                             uint8_t sensorID,
                                             uint8_t cycleID,
                                             size_t size)
{
    DataTable_Error error = DataTable_NoError;

    // Only add packet if it will not exceed the maximum number of entries
    if(table->metadata->totalDataEntries < MAX_NUMBER_OF_PACKETS)
    {
        if((table->metadata->nextAvailableLocation + size) <
           table->metadata->endOfData)
        {
            // Copy packet data to next available location in the packet data section
            uint8_t *destination = memcpy(
                (void*)table->metadata->nextAvailableLocation, packet, size);

            if(destination != NULL)
            {
                // Update metadata if copy was successful
                table->metadata->lookUpTable[table->metadata->totalDataEntries].
                type = type;
                table->metadata->lookUpTable[table->metadata->totalDataEntries].
                sensorID = sensorID;
                table->metadata->lookUpTable[table->metadata->totalDataEntries].
                cycleID = cycleID;
                table->metadata->lookUpTable[table->metadata->totalDataEntries].
                fresh = true;
                table->metadata->lookUpTable[table->metadata->totalDataEntries].
                dataPtr = destination;
                table->metadata->totalDataEntries++;
                table->metadata->nextAvailableLocation = destination + size;
            }
            else
            {
                error = DataTable_MemcpyError;
            }
        }
        else
        {
            error = DataTable_DataByteLimitError;
        }
    }
    else
    {
        error = DataTable_DataPacketLimitError;
    }

    return(error);
}

DataTable_Error CAPT_updatePacketInDataTable(DataTable_Type *table,
                                             uint8_t tableLocation,
                                             void *packet,
                                             size_t size)
{
    DataTable_Error error = DataTable_NoError;

    // Overwrite data in table with new data
    uint8_t *destination =
        memcpy((void*)table->metadata->lookUpTable[tableLocation].dataPtr,
               packet,
               size);

    if(destination == NULL)
    {
        error = DataTable_MemcpyError;
    }
    else
    {
        table->metadata->lookUpTable[tableLocation].fresh = true;
    }

    return(error);
}

uint8_t CAPT_checkForPacketInDataTable(DataTable_Type *table,
                                       DataTable_PacketType type,
                                       uint8_t sensorID,
                                       uint8_t cycleID)
{
    uint8_t lookUpTableLoc = DATATABLE_PACKET_NOT_IN_TABLE_CODE;

    // Check each populated entry in look-up table for desired packet
    uint8_t i;

    for(i = 0; i < table->metadata->totalDataEntries; i++)
    {
        // First try to match type
        if(table->metadata->lookUpTable[i].type == type)
        {
            // Then match sensor ID
            if(table->metadata->lookUpTable[i].sensorID == sensorID)
            {
                // Match cycle
                if(table->metadata->lookUpTable[i].cycleID == cycleID)
                {
                    lookUpTableLoc = i;
                }
            }
        }
    }

    return(lookUpTableLoc);
}
