/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

//*****************************************************************************
// CAPTHost_Interface.c
//*****************************************************************************
#include <CAPTHost_CommConfig.h>
#include <CAPTHost_Interface.h>
#include <stdbool.h>
#include <stdlib.h>
#include "Serial_Drivers/I2CMaster.h"
#include "Serial_Drivers/UART.h"

#if (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)

//! Array used by UART_receiveQueue to store incoming bytes
static uint8_t UART_queueBuffer[CAPT_QUEUE_BUFFER_SIZE];

//! Byte queue consisting of an array, a size, a head pointer and a tail pointer
static ByteQueue_Type UART_receiveQueue;

//! Array used to store data packets
static uint8_t UART_dataPackets[MAX_TOTAL_DATA_BYTES];

//! Metadata for data packet array
static DataTable_Metadata UART_dataPacketMetadata;

//! Data table to store most recent sensor/cycle packets from UART comms
static DataTable_Type UART_dataPacketTable;

#elif (CAPT_INTERFACE == __CAPT_I2C_INTERFACE__)
//! Transmit buffer used for I2C communication
static uint8_t I2C_dataBuffer[CAPT_I2C_REGISTER_RW_BUFFER_SIZE];
#endif

#if (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
static bool CAPT_UARTReceiveHandler(uint8_t data)
{
    // Save received byte in byte queue
    CAPT_pushOntoByteQueue(&UART_receiveQueue, data);
    return(true);
}

static const UART_PortType UARTPort =
{
    // UART interface settings
    .receiveCallback = CAPT_UARTReceiveHandler,
    .errorCallback = NULL,
    .peripheralParameters.selectClockSource = EUSCI_A_UART_CLOCKSOURCE_SMCLK,
    .peripheralParameters.clockPrescalar = UART__PRESCALER,
    .peripheralParameters.firstModReg = UART__FIRST_STAGE_MOD,
    .peripheralParameters.secondModReg = UART__SECOND_STAGE_MOD,
    .peripheralParameters.parity = EUSCI_A_UART_NO_PARITY,
    .peripheralParameters.msborLsbFirst = EUSCI_A_UART_LSB_FIRST,
    .peripheralParameters.numberofStopBits = EUSCI_A_UART_ONE_STOP_BIT,
    .peripheralParameters.uartMode = EUSCI_A_UART_MODE,
    .peripheralParameters.overSampling = UART__SAMPLING_MODE
};
#elif (CAPT_INTERFACE == __CAPT_I2C_INTERFACE__)

static const I2C_PortType I2CPort =
{
    // I2C interface settings
    .peripheralParameters.selectClockSource = EUSCI_B_I2C_CLOCKSOURCE_SMCLK,
    .peripheralParameters.i2cClk = CS_48MHZ,
    .peripheralParameters.dataRate = EUSCI_B_I2C_SET_DATA_RATE_400KBPS,
    .peripheralParameters.byteCounterThreshold = 0,
    .peripheralParameters.autoSTOPGeneration =
        EUSCI_B_I2C_SEND_STOP_AUTOMATICALLY_ON_BYTECOUNT_THRESHOLD,
};

#endif

void CAPT_initCommInterface(void)
{
#if (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
    // Initialize byte queue by associating buffer with queue struct
    // and setting the size
    CAPT_initByteQueue(&UART_receiveQueue, UART_queueBuffer,
                       CAPT_QUEUE_BUFFER_SIZE);
    CAPT_initDataTable(&UART_dataPacketTable, &UART_dataPacketMetadata,
                       UART_dataPackets, MAX_TOTAL_DATA_BYTES);

    // Open the UART port. Initialize with settings in UARTPort struct
    UART_openPort(&UARTPort);
#elif (CAPT_INTERFACE == __CAPT_I2C_INTERFACE__)
    I2CMaster_open(&I2CPort);
#endif
}

SensorData* CAPT_getSensorPacket(uint8_t sensorID)
{
    SensorData *payload = NULL;

#if (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)

    // Look for requested packet in data table
    uint8_t dataTableLookupLocation = CAPT_checkForPacketInDataTable(
        &UART_dataPacketTable, DataTable_SensorPacketType, sensorID, NULL);
    if(dataTableLookupLocation != DATATABLE_PACKET_NOT_IN_TABLE_CODE)
    {
        // Check to make sure data has not already been read
        if(UART_dataPacketTable.metadata->lookUpTable[dataTableLookupLocation].
           fresh)
        {
            // Save data
            payload =
                (SensorData*)UART_dataPacketTable.metadata->lookUpTable[
                    dataTableLookupLocation
                ].dataPtr;
            UART_dataPacketTable.metadata->lookUpTable[dataTableLookupLocation].
            fresh = false;
        }
    }
#elif (CAPT_INTERFACE == __CAPT_I2C_INTERFACE__)

    // Build sensor packet request cmd
    I2C_dataBuffer[CMD_BYTE_INDEX] = SENSOR_PACKET_CMD_BYTE;
    I2C_dataBuffer[SENSOR_ID_INDEX] = sensorID;

    // Request sensor packet
    I2CMasterError error = I2CMaster_writeBuffer(DEFAULT_I2C_SLAVE_ID,
                                                 &I2C_dataBuffer[0],
                                                 SENSOR_PACKET_REQUEST_SIZE);
    if(error == I2CMaster_noError)
    {
        // Read incoming sensor packet into buffer
        error = I2CMaster_readBuffer(DEFAULT_I2C_SLAVE_ID, &I2C_dataBuffer[0],
                                     SENSOR_PACKET_SIZE);
        if(error == I2CMaster_noError)
        {
            // Verify checksum
            if(CAPT_verifyChecksum(I2C_dataBuffer, SENSOR_PACKET_SIZE))
            {
                // Store pointer to buffer
                payload = (SensorData*)I2C_dataBuffer;
            }
        }
    }
#endif
    return(payload);
}

bool CAPT_getCyclePacket(CyclePacket *packet,
                         uint8_t sensorID,
                         uint8_t cycleID,
                         uint8_t numElements)
{
    bool packetFound = false;

#if (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)
    // Look for packet in data table
    uint8_t dataTableLookupLocation = CAPT_checkForPacketInDataTable(
        &UART_dataPacketTable, DataTable_CyclePacketType, sensorID, cycleID);
    if(dataTableLookupLocation != DATATABLE_PACKET_NOT_IN_TABLE_CODE)
    {
        // Check to make sure data has not been read before
        if(UART_dataPacketTable.metadata->lookUpTable[dataTableLookupLocation].
           fresh)
        {
            // Save data in given cycle packet structure
            packetFound = true;
            packet->cyclePacketBase =
                (CycleData*)UART_dataPacketTable.metadata->lookUpTable[
                    dataTableLookupLocation].dataPtr;
            packet->elementData =
                (ElementData*)(UART_dataPacketTable.metadata->lookUpTable[
                                   dataTableLookupLocation].dataPtr +
                               CYCLE_PACKET_BASE_SIZE);
            packet->numElementsOnCycle = numElements;
            UART_dataPacketTable.metadata->lookUpTable[dataTableLookupLocation].
            fresh = false;
        }
    }
#elif (CAPT_INTERFACE == __CAPT_I2C_INTERFACE__)

    // Calculate size of cycle packet
    uint8_t packetSize = CYCLE_PACKET_BASE_SIZE +
                         CYCLE_PACKET_SIZE_PER_ELEMENT * numElements;

    // Build cycle packet request cmd
    I2C_dataBuffer[CMD_BYTE_INDEX] = CYCLE_PACKET_CMD_BYTE;
    I2C_dataBuffer[SENSOR_ID_INDEX] = sensorID;
    I2C_dataBuffer[CYCLE_ID_INDEX] = cycleID;

    // Request cycle packet
    I2CMasterError error = I2CMaster_writeBuffer(DEFAULT_I2C_SLAVE_ID,
                                                 &I2C_dataBuffer[0],
                                                 CYCLE_PACKET_REQUEST_SIZE);
    if(error == I2CMaster_noError)
    {
        // Read incoming data into buffer
        error = I2CMaster_readBuffer(DEFAULT_I2C_SLAVE_ID, &I2C_dataBuffer[0],
                                     packetSize);
        if(error == I2CMaster_noError)
        {
            if(CAPT_verifyChecksum(I2C_dataBuffer, packetSize))
            {
                packetFound = true;
                // Store pointer to buffer
                packet->cyclePacketBase = (CycleData*)I2C_dataBuffer;
                packet->elementData =
                    (ElementData*)(I2C_dataBuffer + CYCLE_PACKET_BASE_SIZE);
                packet->numElementsOnCycle = numElements;
            }
        }
    }
#endif
    return(packetFound);
}

#if (CAPT_INTERFACE == __CAPT_UART_INTERFACE__)

uint8_t* CAPT_getGeneralPurposePacket(void)
{
    uint8_t* dataPtr = NULL;
    // Look for data in table
    uint8_t dataTableLookupLocation = CAPT_checkForPacketInDataTable(
        &UART_dataPacketTable, DataTable_GeneralPacketType, NULL, NULL);

    if(dataTableLookupLocation != DATATABLE_PACKET_NOT_IN_TABLE_CODE)
    {
        // Check to make sure data has not been read before
        if(UART_dataPacketTable.metadata->lookUpTable[dataTableLookupLocation].
           fresh)
        {
            // Save pointer to data
            dataPtr =
                UART_dataPacketTable.metadata->lookUpTable[
                    dataTableLookupLocation].
                dataPtr;
            UART_dataPacketTable.metadata->lookUpTable[dataTableLookupLocation].
            fresh = false;
        }
    }
    return(dataPtr);
}

bool CAPT_checkForInboundPacket(void)
{
    bool packetFound = false;
    // Check byte queue for valid data
    CAPT_CommsError error = CAPT_processReceivedData(&UART_receiveQueue,
                                                     &UART_dataPacketTable);

    if(error == CAPT_NoError)
    {
        packetFound = true;
    }
    return(packetFound);
}

CAPT_CommsError CAPT_processReceivedData(ByteQueue_Type *receiveQueue,
                                         DataTable_Type *table)
{
    GeneralPacketType receivedPacket;
    CAPT_CommsError error = CAPT_NoError;

    // Look for packet in queue and verify checksum
    if(CAPT_parseQueueForPacket(receiveQueue, &receivedPacket))
    {
        DataTable_PacketType type;
        uint8_t sensorID, cycleID;

        // Determine the type of packet
        if(receivedPacket.payload[0] == SENSOR_PACKET_CMD_BYTE)
        {
            type = DataTable_SensorPacketType;
            sensorID = receivedPacket.payload[1];
            cycleID = NULL;
        }
        else if(receivedPacket.payload[0] == CYCLE_PACKET_CMD_BYTE)
        {
            type = DataTable_CyclePacketType;
            sensorID = receivedPacket.payload[1];
            cycleID = receivedPacket.payload[2];
        }
        else if(receivedPacket.payload[0] == GENERAL_PURPOSE_PACKET_CMD_BYTE)
        {
            type = DataTable_GeneralPacketType;
            sensorID = NULL;
            cycleID = NULL;
        }
        else
        {
            error = CAPT_InvalidPacketTypeError;
        }

        if(error == CAPT_NoError)
        {
            // Add or update the data table if a valid packet was found
            if(CAPT_putReceivedPacketInDataTable(table,
                                                 (void*)receivedPacket.payload,
                                                 type, sensorID, cycleID,
                                                 ((size_t)receivedPacket.length
                                                  - 2)) > 0)
            {
                error = CAPT_DataTableError;
            }
        }
    }
    else
    {
        // Checksum not verified, therefore packet not found
        error = CAPT_ChecksumError;
    }

    return(error);
}

#endif
