/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
// TI Release: 1.0.0.00
// Release Date: Friday, August 12, 2016 8:54 AM
//#############################################################################

//*****************************************************************************
//! \addtogroup CAPT_Protocol
//! @{
//*****************************************************************************

#ifndef CAPTIVATE_HOST_COMMS_COMM_CAPT_PROTOCOL_H_
#define CAPTIVATE_HOST_COMMS_COMM_CAPT_PROTOCOL_H_

//****************************************************************************
// Includes
//****************************************************************************
#include <stdint.h>
#include <stdbool.h>
#include "CAPTHost_ByteQueue.h"

//****************************************************************************
// Defines
//****************************************************************************

//! \brief Default address of an I2C slave (used by CapTIvate MCU in slave mode)
#define DEFAULT_I2C_SLAVE_ID    (0x0A)

//*****************************************************************************
//
// CapTIvate&tm; Command Bytes
//
//*****************************************************************************

//! \brief Defines the command byte associated with a sensor packet
#define SENSOR_PACKET_CMD_BYTE                          (0x00)
//! \brief Defines the command byte associated with a cycle packet
#define CYCLE_PACKET_CMD_BYTE                           (0x01)
//! \brief Defines the command byte associated with a general purpose packet
#define GENERAL_PURPOSE_PACKET_CMD_BYTE         (0x10)

//*****************************************************************************
//
// CapTIvate&tm; Protocol Info
//
//*****************************************************************************

//! \brief The index in the CapTIvate&tm; packet of the command byte
#define CMD_BYTE_INDEX  (0)
//! \brief The index in the CapTIvate&tm; packet of the sensor ID byte
#define SENSOR_ID_INDEX (1)
//! \brief The index in the CapTIvate&tm; packet of the cycle ID byte
#define CYCLE_ID_INDEX  (2)

//*****************************************************************************
//
// Sizes of data transmissions with CapTIvate&tm; protocol
//
//*****************************************************************************

//! \brief The number of bytes sent via I2C to request a sensor packet
#define SENSOR_PACKET_REQUEST_SIZE              (2)
//! \brief The size of a CapTIvate&tm; sensor packet
#define SENSOR_PACKET_SIZE                              (8)
//! \brief The number of bytes sent via I2C to request a cycle packet
#define CYCLE_PACKET_REQUEST_SIZE               (3)
//! \brief The size of a cycle packet, excluding element specific data
#define CYCLE_PACKET_BASE_SIZE                  (8)
//! \brief The number of bytes added to a cycle packet for each element on the cycle
#define CYCLE_PACKET_SIZE_PER_ELEMENT   (4)
//! \brief The maximium length of a cycle data packet
#define CYCLE_PACKET_MAX_SIZE                   (25)
//! \brief Defines the maximum length of a CapTIvate&tm; data packet
#define MAX_PACKET_LENGTH                               (62)
//! \brief Defines the length of the checksum in bytes
#define PCKT_CHKSUM_LENGTH                              (2)

//*****************************************************************************
//
// CapTIvate&tm; Protocol Control Bytes
//
//*****************************************************************************

//! \brief Defines the CapTIvate&tm; protocol sync byte
#define PROTOCOL_SYNC                                   (0x55)
//! \brief Defines a blanking byte (can be anything other than the SYNC byte)
#define PROTOCOL_BLANK                                  (0xAA)

//****************************************************************************
// Typedefs
//****************************************************************************

//*****************************************************************************
//
//! typedef GeneralPacketType is the data type used to store CapTIvate&tm;
//! data packets of unkown type
//
//*****************************************************************************
typedef struct
{
    //
    //! payload stores the payload for an unidentified packet type
    //
    uint8_t payload[MAX_PACKET_LENGTH];

    //
    //! length is the number of bytes in the packet
    //
    uint8_t length;
} GeneralPacketType;

//*****************************************************************************
//
//! typedef SensorData is the structure of CapTIvate&tm; sensor data
//
//*****************************************************************************
typedef struct
{
    //
    //! cmdByte is the command byte sent with the data (should always be 0x00)
    //
    uint8_t cmdByte;

    //
    //! sensorIDByte is the ID of the sensor associated with data
    //
    uint8_t sensorIDByte;

    //
    //! byte0 is the first byte in the sensor packet payload. It could represent
    //! dominant element, lower byte of position, etc
    //
    uint8_t byte0;

    //
    //! byte1 is the second byte in the sensor packet payload. It could
    //! represent previous dominant element, upper byte of position, etc.
    uint8_t byte1;

    //
    //! rsvd is a reserved data byte (It should not be used by host)
    //
    uint8_t rsvd;
    union
    {
        //
        //! sensorStatus contains the sensor status flags
        //
        uint8_t sensorStatus;
        struct
        {
            uint8_t touchFlag : 1;
            uint8_t prevTouchFlag : 1;
            uint8_t proximityFlag : 1;
            uint8_t detectFlag : 1;
            uint8_t negTouchFlag : 1;
            uint8_t noiseState : 1;
            uint8_t maxCountErrorFlag : 1;
            uint8_t calibrationErrorFlag : 1;
        };
    };
} SensorData;

//*****************************************************************************
//
//! typedef CycleData is the CapTIvate&tm; cycle packet base structure
//
//*****************************************************************************
//!
typedef struct
{
    //
    //! cmdByte is the command byte sent with the data
    //
    uint8_t cmdByte;

    //
    //! sensorIDByte is the ID of the sensor associated with the data
    //
    uint8_t sensorIDByte;

    //
    //! cycleIDByte is the ID of the cycle associated with the data
    //
    uint8_t cycleIDByte;
    union
    {
        struct
        {
            //
            //! stateBytes contain all touch and proximity flags for elements
            //! on the cycle
            //
            uint8_t stateByte0;
            uint8_t stateByte1;
            uint8_t stateByte2;
        };
        struct
        {
            uint8_t element1Prox : 1;
            uint8_t element2Prox : 1;
            uint8_t element3Prox : 1;
            uint8_t element4Prox : 1;
            uint8_t element5Prox : 1;
            uint8_t element6Prox : 1;
            uint8_t element7Prox : 1;
            uint8_t element8Prox : 1;
            uint8_t element9Prox : 1;
            uint8_t element10Prox : 1;
            uint8_t element11Prox : 1;
            uint8_t element12Prox : 1;
            uint8_t element1Touch : 1;
            uint8_t element2Touch : 1;
            uint8_t element3Touch : 1;
            uint8_t element4Touch : 1;
            uint8_t element5Touch : 1;
            uint8_t element6Touch : 1;
            uint8_t element7Touch : 1;
            uint8_t element8Touch : 1;
            uint8_t element9Touch : 1;
            uint8_t element10Touch : 1;
            uint8_t element11Touch : 1;
            uint8_t element12Touch : 1;
        };
    };
} CycleData;

//*****************************************************************************
//
//! typedef ElementData is the structure of element data included on the end
//! of a cycle packet
//
//*****************************************************************************
typedef struct
{
    union
    {
        struct
        {
            //
            //! LTA is the long term average of the element
            //
            uint16_t LTA;

            //
            //! count is the count on the element
            //
            uint16_t count;
        };
        struct
        {
            uint8_t LTA_L;
            uint8_t LTA_U;
            uint8_t count_L;
            uint8_t count_H;
        };
    };
} ElementData;

//*****************************************************************************
//
//! typedef CyclePacket is the structure of an entire CapTIvate&tm; cycle
//! packet, including the cycle packet base and all associated element data.
//
//*****************************************************************************
typedef struct
{
    //
    //! cyclePacketBase is the fixed set of data associated with a cycle packet
    //
    CycleData *cyclePacketBase;

    //
    //! *elementData is a pointer to the first set of element data received
    //! with a cycle packet
    //
    ElementData *elementData;

    //
    //! numElementsOnCycle is the number of elements on the cycle
    //
    uint8_t numElementsOnCycle;
} CyclePacket;

//****************************************************************************
// Function Prototypes
//****************************************************************************

//*****************************************************************************
//
//! Parses through a byte queue to extract a data packet
//
//! \param receiveQueue - Byte queue to look for data packet in
//! \param packet - Container in which to place extracted packet
//!
//! \returns true if a valid packet is found, else false
//
//*****************************************************************************
bool CAPT_parseQueueForPacket(ByteQueue_Type *receiveQueue,
                              GeneralPacketType *packet);

//*****************************************************************************
//
//! Verifies the checksum of a byte buffer of given length.
//
//! \param data - Array of data bytes for which to verify checksum (includes expected checksum bytes)
//! \param length - Length of array in bytes
//!
//! \returns true if the checksum is valid, else false.
//
//*****************************************************************************
bool CAPT_verifyChecksum(uint8_t *data,
                         uint8_t length);

#endif /* CAPTIVATE_HOST_COMMS_COMM_CAPT_PROTOCOL_H_ */
//*****************************************************************************
// Close the Doxygen group.
//! @}
//*****************************************************************************
