/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
// UART.h
// Created on: Nov 6, 2015
//*****************************************************************************
//*****************************************************************************
//! \addtogroup UART
//! @{
//*****************************************************************************
#ifndef CAPTIVATE_MASTER_COMM_SERIAL_DRIVERS_UART_H_
#define CAPTIVATE_MASTER_COMM_SERIAL_DRIVERS_UART_H_

#include "UART_Definitions.h"

#if (UART__ENABLE == true)

//*****************************************************************************
//
//! def UART__ERROR_MASK defines the error bits from the eUSCI_A peripheral
//! that cause an error callback to occur.  The errors communicated are
//! UCOE and UCFE for overrun errors and framing errors, respectivly.
//
//*****************************************************************************
#define UART__ERROR_MASK                (0x60)

//*****************************************************************************
//
//! \typedef tUARTPort defines the port definition that the user must
//! provide when instantiating this driver.  This structure is read-only from
//! the context of the driver, and may be placed in read-only memory, such as
//! the C CONST section, if desired.
//
//*****************************************************************************
typedef struct
{
    //
    //! Receive Callback.  This function will be called
    //! after each UART byte is received.  This fxn is called
    //! from the foreground (ISR), and as such it should be kept
    //! as short as possible.
    //
    bool (*receiveCallback)(uint8_t);

    //
    //! Error Callback.  This function will be called after
    //! a UART error is detected.  The passed tUARTErrors
    //! parameter indicates what caused the error.
    //
    bool (*errorCallback)(uint8_t);

    //
    //! eUSCI_A initialization parameters.  This structure
    //! is defined in the MSP430 peripheral driver library.
    //
    eUSCI_UART_Config peripheralParameters;
} UART_PortType;

//*****************************************************************************
//
//! \enum tUARTStates enumerates the possible UART driver states.
//
//*****************************************************************************
enum UARTStates
{
    UARTIsClosed,
    UARTIsIdle,
    UARTIsTransmitting,
};

//*****************************************************************************
//
//! \enum tUARTErrors enumerates the possible UART driver errors.
//! These error codes are passed to the error callback function.
//
//*****************************************************************************
enum UARTErrors
{
    UARTFramingError = 0x40,
    UARTOverrunError = 0x20
};

//*****************************************************************************
//
//! This API opens the UART port on the specified HW eUSCI_A peripheral.
//! This API must be called before the port is used.
//
//! \param pPort is a pointer to the UART Port configuration
//! structure.  The user must populate and provide this structure to the
//! port driver.  The data structure must remain at the same address
//! throughout the operation of the driver.
//! \return none
//
//*****************************************************************************
extern void UART_openPort(const UART_PortType *port);

//*****************************************************************************
//
//! This API closes the UART port on the specified HW eUSCI_A peripheral.
//
//!  \par Parameters
//!		none
//!  \par Returns
//!		none
//
//*****************************************************************************
extern void UART_closePort(void);

//*****************************************************************************
//
//! This API returns the status of the UART port when called.  The possible
//! statuses are enumerated under tUARTStates.
//
//!  \par Parameters
//!		none
//! \return tUARTStates enumeration indicating the current status.
//
//*****************************************************************************
extern uint8_t UART_getPortStatus(void);

//*****************************************************************************
//
//! This API reads a single packet from the buffer pointed to.
//
//! \param pBuffer is a pointer to the buffer to transmit
//!  \par Returns
//!		none
//
//*****************************************************************************
extern void UART_readPacketFromBuffer(const uint8_t *buffer);

#endif  /* UART__ENABLE==true */

#endif  /* CAPTIVATE_MASTER_COMM_SERIAL_DRIVERS_UART_H_ */

//*****************************************************************************
//! Close the doxygen group
//! @}
//*****************************************************************************
